/**
 * Used to store mapping between base node and data objects
 * and the prefuse toolkit. Currently this is just supposed 
 * to store map data. 
 * 
 * 
 * @author Sam Heller
 * @version 2/4/07
 */
package map;
import base.*;


public class Map {
	Edge[] edges;
	Node[] nodes;
	private int nodeCount;
	private int edgeCount;
	String title;
	
	/**
	 * Create a new map object with a given title and
	 * an upper limit of 1000 nodes and 5000 edges.
	 * @param t the title of the map
	 */
	public Map(String t){
		title = t;
		//I'm not really sure about having an initial upper limit on 
		//the node and edge count. Theres a much better way to do this 
		//but I'm not exactly sure how, and I wanna focuse on the rest 
		//of the work for now.
		edges = new Edge[500];
		nodes = new Node[100];
		nodeCount = 0;
		edgeCount = 0;
	}
	
	
	/**
	 * Create a new map object with a given title, as 
	 * well as user defined upper limits on the amount
	 * of nodes and edges in the map.
	 * @param t
	 * @param n
	 * @param e
	 */
	public Map(String t, int n, int e){
		title = t;
		edges = new Edge[e];
		nodes = new Node[n];
		nodeCount = 0;
		edgeCount = 0;
	}
	
	/**
	 * Add an already retrieved node object. First checks to make sure
	 * the nodes[] array has the necessary capacity, and increases the array
	 * size by 100 if it doesn't.
	 * 
	 * @param n a node to be added to the map 
	 */
	public void addNode(Node n){
		//check if we've got enough space if not then resize 
		//the node array up 100 (should probably make this
		//step size customizable. 
		int length = java.lang.reflect.Array.getLength(nodes);
		if (length == nodeCount){
			nodes = (Node[])resize(nodes, nodeCount + 100);
		}
		nodes[nodeCount] = n;
		nodeCount++;
	}
	
	/**
	 * Create a node object from the nodeID. First checks to make sure
	 * the nodes[] array has the necessary capacity, and increases the array
	 * size by 100 if it doesn't.
	 * 
	 * @param id a nodeID integer which refers to a pre-existing node in the database
	 */
	public void addNode(int id){
		//check if we've got enough space if not then resize 
		//the node array up 100 (should probably make this
		//step size customizable. 
		int length = java.lang.reflect.Array.getLength(nodes);
		if (length == nodeCount){
			nodes = (Node[])resize(nodes, nodeCount + 100);
		}
		//Load node from the database
		nodes[nodeCount] = new Node(id);
		nodeCount++;
	}
	
	/**
	 * Create a new edge between two nodes. First checks to make sure
	 * the edges[] array has the necessary capacity, and increases the array
	 * size by 500 if it doesn't.
	 * 
	 * @param s starting node (must be a node object)
	 * @param e ending node (must be a node object)
	 * @param d whether or not the edge is directed
	 */
	public void addEdge(Node s, Node e, boolean d){
		//Check if we've got enough space. If not, resize
		//the object array 
		int length = java.lang.reflect.Array.getLength(edges);
		if (length == edgeCount){
			edges = (Edge[])resize(edges, edgeCount + 500);
		}
		
		Edge edge = new Edge(s, e, d);
		edges[edgeCount] = edge;
		edgeCount++;
	}
	
	

	/**
	* Reallocates an array with a new size, and copies the contents
	* of the old array to the new array.
	* @param array  the old array, to be reallocated.
	* @param size   the new array size.
	* @return          A new array with the same contents.
	*/
	private static Object resize(Object array, int size) {
	   int oldSize = java.lang.reflect.Array.getLength(array);
	   Class elementType = array.getClass().getComponentType();
	   Object newArray = java.lang.reflect.Array.newInstance(elementType,size);
	   int preserveLength = Math.min(oldSize,size);
	   
	   if (preserveLength > 0){ System.arraycopy (array,0,newArray,0,preserveLength); }
	   return newArray; 
	   }


}
